@@ -15,6 +15,7 @@
  */
 
 void _orc_debug_init(void);
+void _orc_once_init(void);
 
 /**
  * orc_init:
@@ -34,6 +35,7 @@ orc_init (void)
   ORC_ASSERT(sizeof(OrcExecutor) == sizeof(OrcExecutorAlt));
 
   _orc_debug_init();
+  _orc_once_init();
   orc_opcode_init();
   orc_c_init();
   orc_c64x_c_init();
