@@ -79,6 +79,7 @@ orc_compiler_allocate_register (OrcCompiler *compiler, int data_reg)
 
   /* FIXME on !x86, this is an error */
   ORC_COMPILER_ERROR (compiler, "register overflow");
+  compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
 
   return 0;
 }
@@ -148,6 +149,12 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   compiler = malloc (sizeof(OrcCompiler));
   memset (compiler, 0, sizeof(OrcCompiler));
 
+  if (program->backup_func) {
+    program->code = program->backup_func;
+  } else {
+    program->code = (void *)orc_executor_emulate;
+  }
+
   compiler->program = program;
   compiler->target = target;
   compiler->target_flags = flags;
@@ -259,6 +266,7 @@ orc_compiler_check_sizes (OrcCompiler *compiler)
         ORC_COMPILER_ERROR(compiler, "size mismatch, opcode %s dest[%d] is %d should be %d",
             opcode->name, j, compiler->vars[insn->dest_args[j]].size,
             opcode->dest_size[j]);
+        compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         return;
       }
     }
@@ -270,8 +278,18 @@ orc_compiler_check_sizes (OrcCompiler *compiler)
         ORC_COMPILER_ERROR(compiler, "size mismatch, opcode %s src[%d] is %d should be %d",
             opcode->name, j, compiler->vars[insn->src_args[j]].size,
             opcode->src_size[j]);
+        compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         return;
       }
+      if (opcode->flags & ORC_STATIC_OPCODE_SCALAR && j >= 1 &&
+          compiler->vars[insn->src_args[j]].vartype != ORC_VAR_TYPE_PARAM &&
+          compiler->vars[insn->src_args[j]].vartype != ORC_VAR_TYPE_CONST) {
+        ORC_COMPILER_ERROR(compiler, "opcode %s requires const or param source",
+            opcode->name);
+        compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
+        return;
+
+      }
     }
   }
 }
@@ -289,6 +307,7 @@ orc_compiler_assign_rules (OrcCompiler *compiler)
 
     if (insn->rule == NULL || insn->rule->emit == NULL) {
       ORC_COMPILER_ERROR(compiler, "No rule for: %s", insn->opcode->name);
+      compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
       return;
     }
   }
@@ -326,6 +345,7 @@ orc_compiler_rewrite_vars (OrcCompiler *compiler)
       if (!compiler->vars[var].used) {
         if (compiler->vars[var].vartype == ORC_VAR_TYPE_TEMP) {
           ORC_COMPILER_ERROR(compiler, "using uninitialized temp var");
+          compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
         compiler->vars[var].used = TRUE;
         compiler->vars[var].first_use = j;
@@ -340,20 +360,25 @@ orc_compiler_rewrite_vars (OrcCompiler *compiler)
 
       if (compiler->vars[var].vartype == ORC_VAR_TYPE_SRC) {
         ORC_COMPILER_ERROR(compiler,"using src var as dest");
+        compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
       }
       if (compiler->vars[var].vartype == ORC_VAR_TYPE_CONST) {
         ORC_COMPILER_ERROR(compiler,"using const var as dest");
+        compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
       }
       if (compiler->vars[var].vartype == ORC_VAR_TYPE_PARAM) {
         ORC_COMPILER_ERROR(compiler,"using param var as dest");
+        compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
       }
       if (opcode->flags & ORC_STATIC_OPCODE_ACCUMULATOR) {
         if (compiler->vars[var].vartype != ORC_VAR_TYPE_ACCUMULATOR) {
           ORC_COMPILER_ERROR(compiler,"accumulating opcode to non-accumulator dest");
+          compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
       } else {
         if (compiler->vars[var].vartype == ORC_VAR_TYPE_ACCUMULATOR) {
           ORC_COMPILER_ERROR(compiler,"non-accumulating opcode to accumulator dest");
+          compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
       }
 
@@ -370,6 +395,7 @@ orc_compiler_rewrite_vars (OrcCompiler *compiler)
 #if 0
         if (compiler->vars[var].vartype == ORC_VAR_TYPE_DEST) {
           ORC_COMPILER_ERROR(compiler,"writing dest more than once");
+          compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
 #endif
         if (compiler->vars[var].vartype == ORC_VAR_TYPE_TEMP) {
@@ -427,6 +453,7 @@ orc_compiler_global_reg_alloc (OrcCompiler *compiler)
         break;
       default:
         ORC_COMPILER_ERROR(compiler, "bad vartype");
+        compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         break;
     }
   }
