@@ -661,7 +661,7 @@ BINARY_FL(cmplef, (a <= b) ? (~0) : 0)
 static void
 convfl (OrcOpcodeExecutor *ex, void *user)
 {
-  ex->dest_values[0] = floor(0.5 + ORC_FLOAT_READ(&ex->src_values[0]));
+  ex->dest_values[0] = ORC_FLOAT_READ(&ex->src_values[0]);
 }
 
 static void
@@ -693,9 +693,9 @@ static OrcStaticOpcode opcodes[] = {
   { "mulhsb", mulhsb, NULL, 0, { 1 }, { 1, 1 } },
   { "mulhub", mulhub, NULL, 0, { 1 }, { 1, 1 } },
   { "orb", orb, NULL, 0, { 1 }, { 1, 1 } },
-  { "shlb", shlb, NULL, 0, { 1 }, { 1, 1 } },
-  { "shrsb", shrsb, NULL, 0, { 1 }, { 1, 1 } },
-  { "shrub", shrub, NULL, 0, { 1 }, { 1, 1 } },
+  { "shlb", shlb, NULL, ORC_STATIC_OPCODE_SCALAR, { 1 }, { 1, 1 } },
+  { "shrsb", shrsb, NULL, ORC_STATIC_OPCODE_SCALAR, { 1 }, { 1, 1 } },
+  { "shrub", shrub, NULL, ORC_STATIC_OPCODE_SCALAR, { 1 }, { 1, 1 } },
   { "signb", signb, NULL, 0, { 1 }, { 1 } },
   { "subb", subb, NULL, 0, { 1 }, { 1, 1 } },
   { "subssb", subssb, NULL, 0, { 1 }, { 1, 1 } },
@@ -722,9 +722,9 @@ static OrcStaticOpcode opcodes[] = {
   { "mulhsw", mulhsw, NULL, 0, { 2 }, { 2, 2 } },
   { "mulhuw", mulhuw, NULL, 0, { 2 }, { 2, 2 } },
   { "orw", orw, NULL, 0, { 2 }, { 2, 2 } },
-  { "shlw", shlw, NULL, 0, { 2 }, { 2, 2 } },
-  { "shrsw", shrsw, NULL, 0, { 2 }, { 2, 2 } },
-  { "shruw", shruw, NULL, 0, { 2 }, { 2, 2 } },
+  { "shlw", shlw, NULL, ORC_STATIC_OPCODE_SCALAR, { 2 }, { 2, 2 } },
+  { "shrsw", shrsw, NULL, ORC_STATIC_OPCODE_SCALAR, { 2 }, { 2, 2 } },
+  { "shruw", shruw, NULL, ORC_STATIC_OPCODE_SCALAR, { 2 }, { 2, 2 } },
   { "signw", signw, NULL, 0, { 2 }, { 2 } },
   { "subw", subw, NULL, 0, { 2 }, { 2, 2 } },
   { "subssw", subssw, NULL, 0, { 2 }, { 2, 2 } },
@@ -751,9 +751,9 @@ static OrcStaticOpcode opcodes[] = {
   { "mulhsl", mulhsl, NULL, 0, { 4 }, { 4, 4 } },
   { "mulhul", mulhul, NULL, 0, { 4 }, { 4, 4 } },
   { "orl", orl, NULL, 0, { 4 }, { 4, 4 } },
-  { "shll", shll, NULL, 0, { 4 }, { 4, 4 } },
-  { "shrsl", shrsl, NULL, 0, { 4 }, { 4, 4 } },
-  { "shrul", shrul, NULL, 0, { 4 }, { 4, 4 } },
+  { "shll", shll, NULL, ORC_STATIC_OPCODE_SCALAR, { 4 }, { 4, 4 } },
+  { "shrsl", shrsl, NULL, ORC_STATIC_OPCODE_SCALAR, { 4 }, { 4, 4 } },
+  { "shrul", shrul, NULL, ORC_STATIC_OPCODE_SCALAR, { 4 }, { 4, 4 } },
   { "signl", signl, NULL, 0, { 4 }, { 4 } },
   { "subl", subl, NULL, 0, { 4 }, { 4, 4 } },
   { "subssl", subssl, NULL, 0, { 4 }, { 4, 4 } },
