@@ -56,6 +56,11 @@ orc_target_c_get_asm_preamble (void)
     "#define ORC_SWAP_L(x) ((((x)&0xff)<<24) | (((x)&0xff00)<<8) | (((x)&0xff0000)>>8) | (((x)&0xff000000)>>24))\n"
     "#define ORC_PTR_OFFSET(ptr,offset) ((void *)(((unsigned char *)(ptr)) + (offset)))\n"
     "#define ORC_AS_FLOAT(x) (((union { int i; float f; } *)(&x))->f)\n"
+    "#ifdef __GNUC_PREREQ\n"
+    "#if __GNUC_PREREQ(4,2)\n"
+    "#pragma GCC diagnostic ignored \"-Wstrict-aliasing\"\n"
+    "#endif\n"
+    "#endif\n"
     "/* end Orc C target preamble */\n\n";
 }
 
@@ -65,7 +70,7 @@ orc_compiler_c_get_default_flags (void)
   return 0;
 }
 
-const char *varnames[] = {
+static const char *varnames[] = {
   "d1", "d2", "d3", "d4",
   "s1", "s2", "s3", "s4",
   "s5", "s6", "s7", "s8",
@@ -77,7 +82,7 @@ const char *varnames[] = {
   "t1", "t2", "t3", "t4",
   "t5", "t6", "t7", "t8",
   "t9", "t10", "t11", "t12",
-  "t13", "t14", "t15", "t16"
+  "t13", "t14", "t15", "t16",
 };
 
 static void
@@ -86,7 +91,11 @@ get_varname (char *s, OrcCompiler *compiler, int var)
   if (!(compiler->target_flags & ORC_TARGET_C_NOEXEC)) {
     sprintf(s, "ex->arrays[%d]", var);
   } else {
-    strcpy (s, varnames[var]);
+    if (var < 48) {
+      strcpy (s, varnames[var]);
+    } else {
+      sprintf(s, "t%d", var-32);
+    }
   }
 }
 
